//Quiz.java presents UM quiz to a student on client machine
//Interacts with server through an instance of QuizClient,
//  which supplies password information and quiz questions,
//  and also records scores in a simple database maintained by QuizServer.
//The GUI for this program uses radio buttons from 
//  JDK 1.1 AWT's CheckboxGroup and Checkbox

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;

public class Quiz extends Frame
             implements ItemListener 
{
   private Label questionLabel;
   private CheckboxGroup radioButtons;
   private Checkbox radio1, radio2, radio3, radio4;
   private String question, answer1, answer2, answer3, answer4, rightAnswer;
   
   private Socket client;
   DataInputStream quizInput;   //get quiz questions and answers from file
   DataOutputStream quizOutput;
   private StringBuffer userAnswers;                //record results of quiz   
   private int totalQuestions,totalCorrect;
   
   public Quiz(String serverName)
   {
     super(  "Quiz for The Universal Machine" );
     setBounds(50,200,700,150);         //center the Quiz on screen
     setLayout(new GridLayout(5, 1));   //control layout of questions and Checkbox buttons
     addWindowListener( new CloseQuizWindow() );
      
     openFilesThruSocket(serverName);  //open network connection to QuizServer 
     Passwd p = new Passwd(this);  
   }

   public void openFilesThruSocket(String serverName)
   {  try 
      { client = new Socket( InetAddress.getByName(serverName), 5000 );
        quizInput = new DataInputStream( client.getInputStream() );
        quizOutput = new DataOutputStream( client.getOutputStream() );
      }
      catch ( IOException e ) 
      { e.printStackTrace(); }
   }

   public void runQuiz()    
   {
      question = new String();
      answer1 = new String();
      answer2 = new String();
      answer3 = new String();
      answer4 = new String();
      rightAnswer = new String();
      userAnswers = new StringBuffer();
    
      try //open file and read questions
      {                    
         question = quizInput.readLine(); //gobble up newline
         question = quizInput.readLine().substring(2);
          answer1 = quizInput.readLine().substring(2);
          answer2 = quizInput.readLine().substring(2);
          answer3 = quizInput.readLine().substring(2);
          answer4 = quizInput.readLine().substring(2);
          rightAnswer = quizInput.readLine().substring(2);
      }
      catch ( IOException e )
      { System.err.println("Error while getting quiz data from server\n" + e.toString() );
        System.exit(1);
      }
       
      //Create Label posing a question
      questionLabel = new Label( question );
      Font questionFont = new Font ( "Arial",Font.BOLD,14 );
      questionLabel.setFont( questionFont );
      add( questionLabel ); 
      
      //Create CheckboxGroup to maintain radio buttons
      radioButtons = new CheckboxGroup(); 

      //Create radio buttons and listen for one to get clicked
      radio1 = new Checkbox(answer1 , radioButtons, false );
      radio1.addItemListener( this );   add( radio1 );
      radio2 = new Checkbox( answer2, radioButtons, false );
      radio2.addItemListener( this );   add( radio2 );
      radio3 = new Checkbox( answer3, radioButtons, false );
      radio3.addItemListener( this );    add( radio3 );
      radio4 = new Checkbox( answer4      , radioButtons, false );
      radio4.addItemListener( this );    add( radio4 );
         
      setVisible( true );
      if (answer4.equals(" ")) radio4.setVisible(false);
   }

   public void itemStateChanged( ItemEvent ev )
   {
      String userAnswer = new String();
      if (ev.getSource() == radio1)  { userAnswer = "1 "; radio1.setState(false); }
      else if (ev.getSource() == radio2)  { userAnswer = "2 "; radio2.setState(false); }
      else if (ev.getSource() == radio3)  { userAnswer = "3 "; radio3.setState(false); }
      else if (ev.getSource() == radio4)  { userAnswer = "4 "; radio4.setState(false); }
      
      userAnswers.append(userAnswer);
      if (userAnswer.equals(rightAnswer))                         
          totalCorrect++;
      totalQuestions++;
      
      try //read answer and next set  questions
      { 
         question = quizInput.readLine().substring(2);   
         if (question.equals("# "))
              finishQuiz();
         answer1 = quizInput.readLine().substring(2);
         answer2 = quizInput.readLine().substring(2);
         answer3 = quizInput.readLine().substring(2);
         answer4 = quizInput.readLine().substring(2);
         rightAnswer = quizInput.readLine().substring(2);
       }
      catch ( IOException e )
      { System.err.println("Error while reading quiz.dat\n" + e.toString() );
        System.exit(1);
      }
          
      questionLabel.setText(question); 
      radio1.setLabel(answer1); radio1.setState(false);
      radio2.setLabel(answer2); radio2.setState(false);
      radio3.setLabel(answer3); radio3.setState(false);
      radio4.setLabel(answer4); radio4.setState(false);
      
      if (answer4.equals(" ")) 
             radio4.setVisible(false);
      else radio4.setVisible(true);
  
      repaint(); 
   }

   private void finishQuiz() 
   {  
       radio1.setVisible(false);
       radio2.setVisible(false);
       radio3.setVisible(false);
       radio4.setVisible(false);
       question = "You got " + totalCorrect + " out of " + totalQuestions +" right!";
             questionLabel.setText(question); 
                        
      try 
       { quizOutput.writeUTF("score=" + totalCorrect +"\n");
       } catch ( IOException e ) {}              
     
      repaint();
      long startTimer = System.currentTimeMillis();
      while (System.currentTimeMillis() < startTimer + 5000) ;
      try 
      { client.close(); } 
      catch ( IOException e ) {} 
      System.exit(0);
    }

   public class CloseQuizWindow extends WindowAdapter 
   {  public void windowClosing( WindowEvent e )
      { System.out.println("UM quiz aborted... no score recorded...");
        System.exit( 0 );
      }
   }//CloseQuizWindow

   public static void main(String args[])
   { 
     //Check command-line parameter (should be server name)
     if (args.length < 1 || args.length > 1)
     { System.out.println("Usage: java Quiz <server name>");
        System.exit(1);
     } 
   Quiz q = new Quiz(args[0]   );
   q.runQuiz();
    }

}
